<?php

declare(strict_types=1);

namespace OCA\VibeEToEe\BackgroundJob;

use OCA\VibeEToEe\AppInfo\Application;
use OCA\VibeEToEe\RollbackService;
use OCP\AppFramework\Utility\ITimeFactory;

//use OCP\BackgroundJob\TimedJob;
use OC\BackgroundJob\TimedJob;

use OCP\IConfig;

class RollbackBckJob extends TimedJob {

	private $config;
	private $rollbackService;

    /**
     * RollbackBckJob constructor.
     * @param IConfig $config
     * @param ITimeFactory $time
     * @param RollbackService $rollbackService
     */
	public function __construct(ITimeFactory $time,
                                IConfig $config,
								RollbackService $rollbackService) {
//		parent::__construct($time);
        $this->time = $time;
		$this->config = $config;
		$this->rollbackService = $rollbackService;

		// Run once an hour
		$this->setInterval(60 * 60);
	}

    /**
     * @param $argument
     * @return mixed|void
     * @throws \OCP\Files\InvalidPathException
     * @throws \OCP\Files\NotFoundException
     */
	protected function run($argument) {
		$automaticRollback = $this->config
			->getAppValue(Application::APP_ID, 'automatic_rollback', 'yes');
		if ($automaticRollback !== 'yes') {
			return;
		}

		$automaticRollbackTTL = (int)$this->config
			->getAppValue(Application::APP_ID, 'automatic_rollback_ttl', 60 * 60 * 24);
		$timestamp = $this->time->getTime() - $automaticRollbackTTL;

		$this->rollbackService->rollbackOlderThan($timestamp, 25);
	}
}
