<?php

declare(strict_types=1);

namespace OCA\VibeEToEe\Connector\Sabre;

use OCA\DAV\Connector\Sabre\Directory;
use OCA\VibeEToEe\UserAgentManager;
use OCP\Files\IRootFolder;
use OCP\IRequest;
use OCP\IUserSession;
use Sabre\DAV\INode;
use Sabre\DAV\PropFind;
use Sabre\DAV\Server;

class PropFindPlugin extends APlugin {

	private $userAgentManager;
	private $request;

    /**
     * PropFindPlugin constructor.
     * @param IRootFolder $rootFolder
     * @param IUserSession $userSession
     * @param UserAgentManager $userAgentManager
     * @param IRequest $request
     */
	public function __construct(IRootFolder $rootFolder,
								IUserSession $userSession,
								UserAgentManager $userAgentManager,
								IRequest $request) {
		parent::__construct($rootFolder, $userSession);
		$this->userAgentManager = $userAgentManager;
		$this->request = $request;
	}

    /**
     * @param Server $server
     */
	public function initialize(Server $server) {
		parent::initialize($server);

		$this->server->on('propFind', [$this, 'updateProperty'], 105);
	}

    /**
     * @param PropFind $propFind
     * @param INode $node
     */
	public function updateProperty(PropFind $propFind, INode $node): void {
		if (!$this->isFile($node->getName(), $node)) {
			return;
		}

		$userAgent = $this->request->getHeader('USER_AGENT');
		$supportE2EEncryption = $this->userAgentManager->supportsVibeEToEe($userAgent);
		if (!$supportE2EEncryption && $node instanceof Directory) {
			$isEncrypted = $propFind->get('{http://nextcloud.org/ns}is-encrypted');
			if ($isEncrypted === '1') {
				$propFind->set('{http://owncloud.org/ns}permissions', '', 200);
			}
		}
	}
}
