<?php

declare(strict_types=1);

namespace OCA\VibeEToEe\Controller;

use OCA\VibeEToEe\EncryptionManager;
use OCA\VibeEToEe\Contracts\IMetaDataStorage;
use OCP\AppFramework\Http\DataResponse;
use OCP\AppFramework\OCS\OCSNotFoundException;
use OCP\AppFramework\OCSController;
use OCP\Files\NotFoundException;
use OCP\ILogger;
use OCP\IRequest;

class EncryptionController extends OCSController {

	private $userId;
	private $metaDataStorage;
	private $manager;
	private $logger;

    /**
     * EncryptionController constructor.
     * @param $AppName
     * @param IRequest $request
     * @param $userId
     * @param IMetaDataStorage $metaDataStorage
     * @param EncryptionManager $manager
     * @param ILogger $logger
     */
	public function __construct($AppName,
								IRequest $request,
								$userId,
								IMetaDataStorage $metaDataStorage,
								EncryptionManager $manager,
								ILogger $logger) {
		parent::__construct($AppName, $request);
		$this->userId = $userId;
		$this->metaDataStorage = $metaDataStorage;
		$this->manager = $manager;
		$this->logger = $logger;
	}

    /**
     * @param int $id
     * @return DataResponse
     * @throws OCSNotFoundException
     *
     * @NoAdminRequired
     */
	public function setEncryptionFlag(int $id): DataResponse {
		try {
			$this->manager->setEncryptionFlag($id);
		} catch (NotFoundException $e) {
			throw new OCSNotFoundException($e->getMessage());
		}

		return new DataResponse();
	}

    /**
     * @param int $id
     * @return DataResponse
     * @throws OCSNotFoundException
     *
     * @NoAdminRequired
     */
	public function removeEncryptionFlag(int $id): DataResponse {
		try {
			$this->manager->removeEncryptionFlag($id);
		} catch (NotFoundException $e) {
			throw new OCSNotFoundException($e->getMessage());
		}

		try {
			$this->metaDataStorage->deleteMetaData($this->userId, $id);
		} catch (\Exception $e) {
			$this->logger->logException($e, ['app' => $this->appName]);
		}

		return new DataResponse();
	}
}
