<?php


namespace OCA\VibeEToEe\Controller;

use Exception;
use OCA\VibeEToEe\Contracts\IStorageIdentify;
use OCP\AppFramework\Http\DataResponse;
use OCP\AppFramework\OCS\OCSBadRequestException;
use OCP\AppFramework\OCSController;
use OCP\IRequest;
use OCP\ILogger;
use OCP\IL10N;
use OCP\AppFramework\Http;

class IdentifyController  extends OCSController
{
    private $userId;
    protected $service;
    private $logger;
    private $l10n;

    /**
     * IdentifyController constructor.
     * @param $AppName
     * @param IRequest $request
     * @param $userId
     * @param IStorageIdentify $StorageIdentify
     * @param ILogger $logger
     * @param IL10N $l10n
     */
    public function __construct(
        $AppName,
        IRequest $request,
        $userId,
        IStorageIdentify $StorageIdentify,
        ILogger $logger,
        IL10N $l10n
    )
    {
        parent::__construct($AppName, $request);
        $this->service = $StorageIdentify;
        $this->userId = $userId;
        $this->logger = $logger;
        $this->l10n = $l10n;
    }


    /**
     *
     * @NoAdminRequired
     */
    public function setIdentify()
    {
        try{
            $strData = file_get_contents('php://input');
            $this->service->setIdentify($this->userId, $strData);

		    return new DataResponse([]);

        } catch (Exception $e) {
			$this->logger->logException($e, ['app' => $this->appName]);
			throw new OCSBadRequestException($this->l10n->t('Internal error') . ' ' . $e->getMessage());
		}
    }

    /**
     * @NoAdminRequired
     *
     * @param string $id
     * @return DataResponse
     * @throws OCSBadRequestException
     */
    public function getIdentify(string $id = ''): DataResponse{
        try{
            $data = $this->service->getIdentify(urldecode($id));
		    return new DataResponse($data);

        } catch (Exception $e) {
			$this->logger->logException($e, ['app' => $this->appName]);
			throw new OCSBadRequestException($this->l10n->t('Internal error') . ' ' . $e->getMessage());
		}
    }
}
