<?php

namespace OCA\VibeEToEe\Controller;

use Exception;

use OCA\VibeEToEe\Events\UnlinkedGroupsEvent;
use OCP\AppFramework\Db\DoesNotExistException;
use OCP\AppFramework\OCS\OCSBadRequestException;
use OCP\AppFramework\OCSController;
use OCP\IRequest;
use OCP\AppFramework\Http;
use OCP\AppFramework\Http\DataResponse;
use OCP\AppFramework\Controller;
use OCP\EventDispatcher\IEventDispatcher;

use OCA\VibeEToEe\Db\GroupMap;
use OCA\VibeEToEe\Db\GroupMapMapper;
use OCP\IL10N;

class LinksController extends OCSController
{

    private $mapper;
    private $userId;
	protected $eventDispatcher;
    private $l10n;

    public function __construct(string $AppName,
                                IRequest $request,
                                GroupMapMapper $mapper,
                                $UserId,
								IEventDispatcher $eventDispatcher,
                                IL10N $l10n)
    {
        parent::__construct($AppName, $request);
        $this->mapper = $mapper;
        $this->userId = $UserId;
		$this->eventDispatcher = $eventDispatcher;
        $this->l10n = $l10n;
    }

    /**
     * @return DataResponse
     * @throws OCSBadRequestException
     */
    public function index()
    {
        try{
            return new DataResponse($this->mapper->getAll());

        } catch (\Exception $e){
            throw new OCSBadRequestException(
                $this->l10n->t('Internal error') . '. ' . $e->getMessage(), $e);
        }
    }

    /**
     * @param string $group
     * @param mixed $vibegroup
     * @return DataResponse
     */
    public function linksCreate($group, $vibegroup)
    {
        try {
            $groupMap = new GroupMap();
            $groupMap->setGroupId((string)$group);
            $groupMap->setVibeId((string)$vibegroup);
            return new DataResponse($this->mapper->insert($groupMap));

        } catch (Exception $e) {
            return new DataResponse([], Http::STATUS_BAD_REQUEST);
        }
    }

    /**
     * @param $group
     * @param $vibegroup
     * @return DataResponse
     * @throws OCSBadRequestException
     */
    public function linksRemove($group_id, $vibe_id)
    {
        try {
            $groupMap = $this->mapper->find($group_id, $vibe_id);
            $this->mapper->delete($groupMap);

		    $this->eventDispatcher->dispatchTyped(new UnlinkedGroupsEvent($group_id, $vibe_id));

            return new DataResponse([], Http::STATUS_ACCEPTED);

        } catch (Exception $e) {
            return new DataResponse($e->getMessage(), Http::STATUS_BAD_REQUEST);
        }
    }
}

