<?php

namespace OCA\VibeEToEe\Controller;

use Exception;

use OCA\VibeEToEe\Contracts\IExportVibe;
use OCA\VibeEToEe\Events\UnlinkedGroupsEvent;
use OCP\AppFramework\OCS\OCSBadRequestException;
use OCP\AppFramework\OCSController;
use OCP\IRequest;
use OCP\AppFramework\Http;
use OCP\AppFramework\Http\DataResponse;

use OCA\VibeEToEe\Db\NeUsersMapper;
use OCP\IL10N;

class NeusersController extends OCSController
{

    private $mapper;
    private $userId;
	protected $eventDispatcher;
    private $l10n;

    /** @var IExportVibe */
    private $exportVibe;

    public function __construct(string $AppName,
                                IRequest $request,
                                $UserId,
                                NeUsersMapper $mapper,
                                IL10N $l10n,
                                IExportVibe $exportVibe)
    {
        parent::__construct($AppName, $request);
        $this->userId = $UserId;
        $this->mapper = $mapper;
        $this->l10n = $l10n;
        $this->exportVibe = $exportVibe;
    }

    /**
     * @param $page
     * @return DataResponse
     * @throws OCSBadRequestException
     */
    public function index(int $page)
    {
        try{
            $r = $this->mapper->paginate($page);
            return new DataResponse($r);

        } catch (\Exception $e){
            throw new OCSBadRequestException(
                $this->l10n->t('Internal error') . '. ' . $e->getMessage(), $e);
        }
    }

    /**
     * @param int $offset
     * @return DataResponse
     * @throws OCSBadRequestException
     */
    public function exportBystep(int $offset){
        try{
            $response = $this->exportVibe
                ->setOffset($offset)
                ->usersListPrepareByStep();

            // 'page'
            // 'totalRows'
            // 'totalPages'
            return new DataResponse($response);


        } catch (\Exception $e){
            throw new OCSBadRequestException($e->getMessage());
        }
    }

    /**
     * @param $selected
     * @return DataResponse
     * @throws OCSBadRequestException
     */
    public function exportSelected($selected){
        try{
            $result = $this->exportVibe->usersListPrepare($selected);
            return new DataResponse();

        } catch (\Exception $e){
            throw new OCSBadRequestException($e->getMessage());
        }
    }
}

