<?php

namespace OCA\VibeEToEe\Controller;

use OCA\VibeEToEe\AppInfo\Application;
use OCP\AppFramework\Http\DataResponse;
use OCP\AppFramework\OCS\OCSBadRequestException;
use OCP\AppFramework\OCSController;
use OCP\IConfig;
use OCP\IRequest;
use OCA\VibeEToEe\Contracts\IRestApi;
use OCP\IL10N;
use Psr\Log\LoggerInterface;

class SetupController extends OCSController {
    /** @var IConfig */
    protected $config;

    /** @var IRestApi */
    protected $restapi;

    private $logger;
    private $l10n;

    public function __construct($appName,
                                IRequest $request,
                                IRestApi $restapi,
                                IConfig $config,
                                LoggerInterface $logger,
                                IL10N $l10n
                                )
    {
        parent::__construct($appName, $request);
        $this->logger = $logger;
        $this->config = $config;
        $this->restapi = $restapi;
        $this->l10n = $l10n;
    }

    /**
     * @param string $identity
     * @return DataResponse
     * @throws OCSBadRequestException
     */
    public function setSetupApikey(string $apikey){
        $_apikey = $this->config->getAppValue(Application::APP_ID, 'vibe-apikey');
        if(empty($apikey))
            throw new OCSBadRequestException($this->l10n->t('API key should not be empty'));

        try{
            $this->config->setAppValue(Application::APP_ID, 'vibe-apikey', $apikey);
            $this->restapi->chkAccessApiKey();
		    return new DataResponse([
                "apikey" => $apikey
            ]);

        } catch (\Exception $e){
            $this->config->setAppValue(Application::APP_ID, 'vibe-apikey', $_apikey);
            $this->logger->error($e->getMessage(), ['app' => $this->appName]);
			throw new OCSBadRequestException(
			    $this->l10n->t('Internal error') . '. ' . $e->getMessage(), $e);
        }
    }

    /**
     * @return DataResponse
     * @throws OCSBadRequestException
     */
    public function getSetupApikey(){
        try{
            $apikey = $this->config->getAppValue(Application::APP_ID, 'vibe-apikey');
            $regUrl = $this->restapi->configVibe->urlVibeRegister ?? '#';

            $setup = [
                "apikey" => $apikey,
                "regUrl" => $regUrl,
            ];
            return new DataResponse($setup);


        } catch (\Exception $e){
            $this->logger->logException($e, ['app' => $this->appName]);
            throw new OCSBadRequestException($this->l10n->t('Internal error') . ' ' . $e->getMessage());
        }
    }

    /**
     * @return DataResponse
     * @throws OCSBadRequestException
     */
    public function getSetupTimeout(){
        try{
            $timeout = $this->config->getAppValue(Application::APP_ID, 'vibe-timeout', 120);

            $setup = [
                "timeout" => $timeout,
            ];
            return new DataResponse($setup);


        } catch (\Exception $e){
            $this->logger->logException($e, ['app' => $this->appName]);
            throw new OCSBadRequestException($this->l10n->t('Internal error') . ' ' . $e->getMessage());
        }
    }

    /**
     * @param int $timeout
     * @return DataResponse
     * @throws OCSBadRequestException
     */
    public function setSetupTimeout(int $timeout){
        $_timeout = $this->config->getAppValue(Application::APP_ID, 'vibe-timeout', 120);
        if(empty($timeout) || $timeout == 0)
            throw new OCSBadRequestException($this->l10n->t('Timeout should not be empty'));

        try{
            $this->config->setAppValue(Application::APP_ID, 'vibe-timeout', $timeout);
            return new DataResponse([
                "timeout" => $timeout
            ]);

        } catch (\Exception $e){
            $this->config->setAppValue(Application::APP_ID, 'vibe-timeout', $_timeout);
            $this->logger->error($e->getMessage(), ['app' => $this->appName]);
            throw new OCSBadRequestException(
                $this->l10n->t('Internal error') . '. ' . $e->getMessage(), $e);
        }
    }
}
