<?php

declare(strict_types=1);

namespace OCA\VibeEToEe\Controller;

use Exception;
use OCA\VibeEToEe\Db\GroupMapMapper;
use OCA\VibeEToEe\Service\UserAdminService as UAservice;
use OCP\AppFramework\OCS\OCSBadRequestException;
use OCP\ILogger;
use OCP\AppFramework\OCSController;
use OCP\AppFramework\Http\JSONResponse;
use OCP\IRequest;
use OCA\VibeEToEe\Service\RestApi;

use OCP\AppFramework\OCS\OCSException;
use OCP\AppFramework\OCS\OCSNotFoundException;
use OCP\AppFramework\Http\DataResponse;
use OC\User\NoUserException;
use OC_Helper;
use OCP\IL10N;

class UsersController extends OCSController
{
    private $logger;
	protected $uaservice;
    private $groupMapper;
    private $restApi;


    public function __construct($AppName,
                                IRequest $request,
                                ILogger $logger,
                                UAservice $uaservice,
                                GroupMapMapper $groupMapper,
                                RestApi $restApi
                    )
    {
        parent::__construct($AppName, $request);
        $this->logger = $logger;
		$this->uaservice = $uaservice;
		$this->groupMapper = $groupMapper;
		$this->restApi = $restApi;
    }

    /**
     *
     * @return JSONResponse
     */
    public function getListSharedUsersPerGroup()
    {
        return new JSONResponse($this->mockupData());
    }

    /**
     * @NoCSRFRequired
     *
     */
    public function getInfoUser(string $userId){
		$data = [];
        return new JSONResponse($data);
    }


    /**
     * @NoAdminRequired
     * @NoCSRFRequired
     *
     * @return JSONResponse
     */
    public function responseVibe(){
        $jsonStr = file_get_contents('php://input');
        $jsonObj = json_decode($jsonStr);

        if(empty($jsonObj->status) || empty($jsonObj->data)){
            return new JSONResponse([
                'status' => 'error',
                'message' => 'Incorrect data'
                ]);
        }

        if($jsonObj->status === 'error')
            $this->logger->error('Vibe admin registration: ' . $jsonObj->message);

        try{
            if($jsonObj->status === 'success'){
                $this->uaservice->chkResponseVibe($jsonObj->data->userId, $jsonObj->data->passPhrase, $jsonObj->data->vibeKey);
                $this->logger->info('Vibe admin registration for User ID: ' . $jsonObj->data->userId . ' SUCCESS');
            }
            return new JSONResponse(['status' => 'success']);

        } catch (\Exception $e){
            return new JSONResponse([
                'status' => 'error',
                'message' => $e->getMessage()
                ]);
        }
    }

    /**
     * @NoAdminRequired
     * @NoCSRFRequired
     *
     * @param string $userId
     * @param string $passPhrase
     * @return JSONResponse
     */
    public function resetVibeAdmin(string $userId, string $passPhrase){
        try{
            $this->uaservice->resetVibeAdmin($userId, $passPhrase);
            $this->logger->info('Vibe admin reset for User ID: ' . $userId);
            return new JSONResponse(['status' => 'success']);

        } catch (\Exception $e){
            return new JSONResponse([
                'status' => 'error',
                'message' => $e->getMessage()
                ]);
        }
    }

    /**
     * @param string $id
     * @return DataResponse
     * @throws OCSBadRequestException
     */
    public function getGroups(string $id = ''): DataResponse{
        try{
//            $result = $this->restApi->setEndpoint('/vibeGetGroupListPerUserid/?userid=' . $id)
//                ->setEndpointMETHOD('GET')
//                ->run();
//            {  “<groupname1>”:”<TC_ID1>”, “<groupname2>”:”<TC_ID2>”, …, “<groupnamen>”:”<TC_IDn>”  }
//            $user = $this->uaservice->getInfoUser(urldecode($id));
//            $groups = $this->groupMapper->findByNcGroups($user['groups']);

            $result = require_once(__DIR__ . '/../../testdata2.php');

            $groupsList = [];
            foreach ($result as $groupName => $tcId){
                $groupsList[] = [
                    'name' => $groupName,
                    'TC_ID' => $tcId
                ];
            }
		    return new DataResponse($groupsList);

        } catch (Exception $e) {
			$this->logger->logException($e, ['app' => $this->appName]);
			throw new OCSBadRequestException($this->l10n->t('Internal error') . ' ' . $e->getMessage());
		}
    }

    /**
     * @return array
     */
    protected function mockupData()
    {
        $users = require_once(__DIR__ . '/../../testdata3.php');
        return $users;
    }

}
