<?php

namespace OCA\VibeEToEe\Controller;

use OCA\VibeEToEe\AppInfo\Application;
use OCA\VibeEToEe\Contracts\IRestApi;
use OCP\AppFramework\Http\DataResponse;
use OCP\AppFramework\OCS\OCSBadRequestException;
use OCP\AppFramework\OCSController;
use OCP\IConfig;
use OCP\IRequest;
use OCP\IUserSession;
use OCP\IGroupManager;
use OCP\IL10N;

class VibeApiController extends OCSController {
	protected $appName;

	/** @var IConfig */
	private $config;

	/** @var IUserSession */
	private $userSession;

	/** @var IGroupManager */
	private $groupManager;

	/** @var bool */
	private $isAdmin;

    /** @var IRestApi */
	private $restApi;

    private $l10n;

	public function __construct(IRequest $request,
								IConfig $config,
								IGroupManager $groupManager,
								IUserSession $userSession,
                                IRestApi $restApi,
                                IL10N $l10n) {
		parent::__construct(Application::APP_ID, $request);

		$this->config = $config;
		$this->appName = Application::APP_ID;
		$this->groupManager = $groupManager;
		$this->userSession = $userSession;
		$this->isAdmin = true;
		$this->restApi = $restApi;
        $this->l10n = $l10n;
	}

	/**
	 * @NoAdminRequired
	 * @NoCSRFRequired
	 *
	 * @returns {JSONResponse} an empty JSONResponse with respective http status code
	 */
	public function getGroupsUser(){
	    try{
	        $user = $this->userSession->getUser();
	        $uid = $user->getUID();

	        /* GROUPS */
            $groupsInfo = new \OC\Group\MetaData(
                $uid,
                $this->isAdmin,
                $this->groupManager,
                $this->userSession
            );

            list($adminGroup, $groups) = $groupsInfo->get();

            return new DataResponse(array_merge_recursive($adminGroup, $groups));
//            return new JSONResponse(array_merge_recursive($adminGroup, $groups), Http::STATUS_OK);


        } catch (\Exception $e){
            throw new OCSBadRequestException(
                $this->l10n->t('Internal error') . '. ' . $e->getMessage(), $e);
        }
    }

    /**
     * @return DataResponse
     * @throws OCSBadRequestException
     */
    public function getGroupsVibe(){
        try{
            $result = json_decode(
                $this->restApi
                ->setEndpoint('getGroups')
                ->run()
            );

            return new DataResponse($result);

        } catch (\Exception $e){
            throw new OCSBadRequestException(
                $this->l10n->t('Internal error') . '. ' . $e->getMessage(), $e);
        }
    }
}
