<?php

namespace OCA\VibeEToEe\Db;

use OC\OCS\Exception;
use OCP\IDBConnection;
use OCP\AppFramework\Db\QBMapper;
use OCP\DB\QueryBuilder\IQueryBuilder;

class ExportMapper extends QBMapper
{

    public function __construct(IDBConnection $db)
    {
        parent::__construct($db, 'vibee2ee_export_queue');
    }


    public function userFind(string $userId)
    {
        try {
            $qb = $this->db->getQueryBuilder();
            $qb->select('*')
                ->from($this->getTableName())
                ->where(
                    $qb->expr()->eq('identity', $qb->createNamedParameter((string)$userId))
                );
            return $this->findEntity($qb);
        } catch (\Exception $e) {
            return null;
        }
    }

    /**
     * @param bool $onlyActive
     * @param int $perUser
     * @return \OCP\AppFramework\Db\Entity[]
     * @throws Exception
     */
    public function getUsers(bool $onlyActive = false, int $perUser = 2)
    {
        try {
            $qb = $this->db->getQueryBuilder();

            $qb->select('*')
                ->from($this->getTableName())
                ->where(
                    $qb->expr()->eq('status', $qb->createNamedParameter($onlyActive))
                )
                ->setMaxResults($perUser);
            return $this->findEntities($qb);

        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * @param bool $onlyActive
     * @return \OCP\AppFramework\Db\Entity[]
     * @throws Exception
     */
    public function getAllUsers(bool $onlyActive = false)
    {
        try {
            $qb = $this->db->getQueryBuilder();

            if (!$onlyActive) {
                $qb->select('*')
                    ->from($this->getTableName())
                    ->where(
                        $qb->expr()->eq('status', $qb->createNamedParameter($onlyActive))
                    );
            } else {
                $qb->select('*')
                    ->from($this->getTableName());
            }
            return $this->findEntities($qb);

        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * @param array $ids
     * @param bool $onlyIds
     * @return array|\OCP\AppFramework\Db\Entity[]
     */
    public function getAllUsersByIdentity(array $ids, bool $onlyIds = true)
    {
        $qb = $this->db->getQueryBuilder();
        $qb->select('*')
            ->from($this->getTableName())
            ->andWhere($qb->expr()->in('identity', $qb->createNamedParameter($ids, IQueryBuilder::PARAM_STR_ARRAY)));

        $result = $this->findEntities($qb);

        if ($onlyIds) {
            $_ids = [];
            while ($result) {
                $i = array_shift($result);
                $_ids[] = $i->getIdentity();
            }
            return $_ids;

        } else {
            return $result;
        }
    }

    /**
     * @param bool $onlyActive
     * @return int
     * @throws Exception
     */
    public function countUsers(bool $onlyActive = false)
    {
        try {
            $qb = $this->db->getQueryBuilder();

            if ($onlyActive) {
                $qb->select($qb->func()->count('*'))
                    ->from($this->getTableName())
                    ->where(
                        $qb->expr()->eq('status', $qb->createNamedParameter(false))
                    );
            } else {
                $qb->select($qb->func()->count('*'))
                    ->from($this->getTableName());
            }
            return (int)$qb->execute()->fetchColumn();

        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * @param array $data
     * @throws \Exception
     */
    public function userCreate(array $data)
    {
        try {
            $user = new Export();
            $user->setIdentity($data['identity']);
            $user->setEmail($data['email'] ?? $data['identity']);
            $user->setData($data['data']);
            $user->setStatus(0);
            $this->insert($user);

        } catch (\Exception $e) {
//            die('<pre>' . print_r($data, true) . '</pre>');
            throw $e;
        }
    }

    /**
     * @return bool
     * @throws Exception
     */
    public function clearTable()
    {
        try {
            $qb = $this->db->getQueryBuilder();
            $sql = $this->db
                ->getDatabasePlatform()
                ->getTruncateTableSQL($qb->getTableName($this->getTableName()));

            return $this->db->prepare($sql)->execute();

        } catch (Exception $e) {
            throw $e;
        }
    }

    /**
     * @return bool
     * @throws Exception
     */
    public function userProcessed($identity)
    {
        try {
            $_type = gettype($identity);
            $qb = $this->db->getQueryBuilder();
            $qb->update($this->tableName)
                ->set('status', $qb->createNamedParameter(true));
            if($_type === 'string') {
                $qb->where(
                    $qb->expr()->eq('identity', $qb->createNamedParameter($identity))
                );
            } elseif ($_type === 'array'){
                $qb->where($qb->expr()->in('identity', $qb->createNamedParameter($identity, IQueryBuilder::PARAM_STR_ARRAY)));
            }

            $qb->execute();
            return true;

        } catch (Exception $e) {
            throw $e;
        }
    }

}
