<?php


namespace OCA\VibeEToEe\Db;

use OCP\AppFramework\Db\DoesNotExistException;
use OCP\IDBConnection;
use OCP\AppFramework\Db\QBMapper;
use OCP\DB\QueryBuilder\IQueryBuilder;

class GroupMapMapper extends QBMapper
{

    public function __construct(IDBConnection $db)
    {
        parent::__construct($db, 'vibee2ee_groups_mapping');
    }

    /**
     * @param string $groupId
     * @param string $vibeId
     * @return \OCP\AppFramework\Db\Entity
     * @throws \OCP\AppFramework\Db\DoesNotExistException
     * @throws \OCP\AppFramework\Db\MultipleObjectsReturnedException
     */
    public function find(string $groupId, string $vibeId)
    {
        $qb = $this->db->getQueryBuilder();

        $qb->select('*')
            ->from($this->getTableName())
            ->where(
                $qb->expr()->eq('group_id', $qb->createNamedParameter($groupId))
            )->andWhere(
                $qb->expr()->eq('vibe_id', $qb->createNamedParameter($vibeId))
            );

        return $this->findEntity($qb);
    }

    /**
     * @param array $groupIds
     * @return \OCP\AppFramework\Db\Entity
     * @throws \OCP\AppFramework\Db\DoesNotExistException
     * @throws \OCP\AppFramework\Db\MultipleObjectsReturnedException
     */
    public function findByNcGroups(array $groupIds){

        $qb = $this->db->getQueryBuilder();
		$qb
			->select('*')
			->from($this->tableName)
			->where($qb->expr()->in('group_id', $qb->createNamedParameter($groupIds, IQueryBuilder::PARAM_STR_ARRAY)));

		return $this->findEntities($qb);

    }

    /**
     * @param string $groupId
     * @param string $vibeId
     * @return false|\OCP\AppFramework\Db\Entity
     * @throws \OCP\AppFramework\Db\MultipleObjectsReturnedException
     */
    public function linkIssue(string $groupId, string $vibeId){
        $qb = $this->db->getQueryBuilder();

        $qb->select('*')
            ->from($this->getTableName())
            ->where(
                $qb->expr()->eq('group_id', $qb->createNamedParameter($groupId))
            )->orWhere(
                $qb->expr()->eq('vibe_id', $qb->createNamedParameter($vibeId))
            );

        try{
            return $this->findEntity($qb);

        } catch (DoesNotExistException $e){
            return false;
        }

    }

    /**
     * @return array|\OCP\AppFramework\Db\Entity[]
     */
    public function getAll()
    {
        $qb = $this->db->getQueryBuilder();

        $qb->select('*')
            ->from($this->getTableName());

        return $this->findEntities($qb);
    }

    /**
     * @param $group
     * @return bool
     * @throws \OCP\AppFramework\Db\DoesNotExistException
     * @throws \OCP\AppFramework\Db\MultipleObjectsReturnedException
     */
    public function createLink($group){
        $is = $this->linkIssue($group, $group);
        if($is)
            return false;

        $groupMap = new GroupMap();
        $groupMap->setGroupId((string)$group);
        $groupMap->setVibeId((string)$group);
        $this->insert($groupMap);
        return true;
    }
}
