<?php


namespace OCA\VibeEToEe\Db;

use Exception;
use OCP\IDBConnection;
use OCP\AppFramework\Db\QBMapper;
use OCP\DB\QueryBuilder\IQueryBuilder;

class NeUsersMapper extends QBMapper
{

    public function __construct(IDBConnection $db)
    {
        parent::__construct($db, 'vibee2ee_users_admin');
    }

    /**
     * @param int $page
     * @param int $perPage
     * @return array
     * @throws \OCP\AppFramework\Db\DoesNotExistException
     * @throws \OCP\AppFramework\Db\MultipleObjectsReturnedException
     */
    public function paginate(int $page = 1, int $perPage = 50)
    {
        try{
            $totalRows = $this->countUsers();

            $totalPages = ceil($totalRows / $perPage);

            $qb = $this->db->getQueryBuilder();
            $qb->select('*')
                ->from($this->getTableName())
                ->where(
                    $qb->expr()->eq('vibe_key', $qb->createNamedParameter(''))
                )
                ->orWhere(
                    $qb->expr()->isNull('vibe_key')
                )
                ->setMaxResults($perPage)
                ->setFirstResult(($page - 1) * $perPage)
            ;

            return [
                'data'          => $this->findEntities($qb),
                'page'          => $page,
                'totalRows'     => $totalRows,
                'totalPages'    => $totalPages,
            ];

        } catch (\Exception $e){
            throw $e;
        }
    }

    /**
     * @param bool $onlyActive
     * @return int
     * @throws Exception
     */
    public function countUsers()
    {
        try {
            $qb = $this->db->getQueryBuilder();
            $qb->select($qb->func()->count('*'))
                ->from($this->getTableName())
                ->where(
                        $qb->expr()->eq('vibe_key', $qb->createNamedParameter(''))
                    )
                ->orWhere(
                    $qb->expr()->isNull('vibe_key')
                )
            ;

            return (int)$qb->execute()->fetchColumn();

        } catch (Exception $e) {
            throw $e;
        }
    }
}
