<?php

namespace OCA\VibeEToEe\Db;

use OCP\IDBConnection;
use OCP\AppFramework\Db\QBMapper;
use OCP\DB\QueryBuilder\IQueryBuilder;

class UserAdminMapper extends QBMapper {

    public function __construct(IDBConnection $db) {
        parent::__construct($db, 'vibee2ee_users_admin');
    }

    public function userFind(string $userId)
    {
        try{
            $qb = $this->db->getQueryBuilder();
            $qb->select('*')
                ->from($this->getTableName())
                ->where(
                    $qb->expr()->eq('user_id', $qb->createNamedParameter((string)$userId))
                )
            ;
            return  $this->findEntity($qb);

        } catch (\Exception $e){
            return null;
        }
    }

    /**
     * @param int $userId
     * @return \OCP\AppFramework\Db\Entity|null
     */
    public function userFindById(int $userId)
    {
        try{
            $qb = $this->db->getQueryBuilder();
            $qb->select('*')
                ->from($this->getTableName())
                ->where(
                    $qb->expr()->eq('id', $qb->createNamedParameter((int)$userId))
                )
            ;
            return  $this->findEntity($qb);

        } catch (\Exception $e){
            return null;
        }
    }

    /**
     * @param string $userId
     * @return UserAdmin|\OCP\AppFramework\Db\Entity
     * @throws \Exception
     */
    public function userFindOrCreate(string $userId)
    {
        try{
            $user = $this->userFind($userId);
            if(!$user){
                $user = new UserAdmin();
                $user->setUserId((string) $userId);
//                $user->setPassPhrase($this->generatePassPhrase());
                $user = $this->insert($user);
            }
            return $user;

        } catch (\Exception $e) {
            throw $e;
        }
    }

    /**
     * @param string $userId
     * @return bool
     * @throws \Exception
     */
    public function userDelete(string $userId)
    {
        try{
            $user = $this->userFind($userId);
            if($user){
                $this->delete($user);
                return true;
            }
            return false;

        } catch (\Exception $e) {
            throw $e;
        }
    }

    /**
     * @param string $userId
     * @param string $vibeKey
     * @return bool
     * @throws \Exception
     */
    public function userCreateToken(string $userId, string $vibeKey)
    {
        try{
            $user = $this->userFind($userId);
            if(!$user) {
                $user = new UserAdmin();
            }

            $user->setVibeKey((string) $vibeKey);
            $user->setUserId((string) $userId);

            if($user->getId())
                $this->update($user);
            else
                $this->insert($user);

            return true;

        } catch (\Exception $e) {
            throw $e;
        }
    }

    /**
     * @param string $groupId
     * @param string $vibeId
     * @return \OCP\AppFramework\Db\Entity
     * @throws \OCP\AppFramework\Db\DoesNotExistException
     * @throws \OCP\AppFramework\Db\MultipleObjectsReturnedException
     */
    public function linkIssue(string $groupId, string $vibeId){
        $qb = $this->db->getQueryBuilder();

        $qb->select('*')
            ->from($this->getTableName())
            ->where(
                $qb->expr()->eq('group_id', $qb->createNamedParameter($groupId))
            )->orWhere(
                $qb->expr()->eq('vibe_id', $qb->createNamedParameter($vibeId))
            );

        return $this->findEntity($qb);
    }

    /**
     * @return array|\OCP\AppFramework\Db\Entity[]
     */
    public function getAll()
    {
        $qb = $this->db->getQueryBuilder();

        $qb->select('*')
            ->from($this->getTableName());

        return $this->findEntities($qb);
    }

    /**
     * @param array $ids
     * @param bool $onlyIds
     * @param bool $withoutKey
     * @return \OCP\AppFramework\Db\Entity[]
     */
    public function getAllByIds(array $ids, bool $onlyIds = true, bool $withoutKey = false)
    {
        $qb = $this->db->getQueryBuilder();

        $qb->select('*')
            ->from($this->getTableName())
            ->andWhere($qb->expr()->in('user_id', $qb->createNamedParameter($ids, IQueryBuilder::PARAM_STR_ARRAY)))
        ;

        if($withoutKey){
            $qb->andWhere(
                $qb->expr()->isNotNull('vibe_key')
            );
        }

        $result =  $this->findEntities($qb);

        if($onlyIds){
            $_ids = [];
            while($result){
                $i = array_shift($result);
                $_ids[] = $i->getUserId();
            }
            return $_ids;

        } else {
            return $result;
        }

    }



    /**
     * @return string
     */
    protected function generatePassPhrase(){
        return md5(uniqid(rand(), true));
    }
}
