<?php

namespace OCA\VibeEToEe\Listener;

use Exception;
use OCA\VibeEToEe\AppInfo\Application;
use OCP\Group\Events\GroupDeletedEvent;
use OCP\IGroupManager;
use Psr\Log\LoggerInterface;
use OCA\VibeEToEe\Db\GroupMapMapper;
use OCP\EventDispatcher\Event;
use OCP\EventDispatcher\IEventListener;
use OCA\VibeEToEe\Contracts\IRestApi;

class GroupDeletedListener implements IEventListener {

    /** @var GroupMapMapper */
    private $mapper;

    /** @var IRestApi */
    private $restApi;

    /** @var LoggerInterface */
    private $logger;

    /** @var IGroupManager */
    private $groupManager;

    /**
     * GroupDeletedListener constructor.
     * @param GroupMapMapper $mapper
     * @param LoggerInterface $logger
     * @param IRestApi $restApi
     * @param IGroupManager $groupManager
     */
	public function __construct(GroupMapMapper $mapper,
                                LoggerInterface $logger,
                                IRestApi $restApi,
                                IGroupManager $groupManager) {
        $this->mapper = $mapper;
        $this->logger = $logger;
        $this->restApi = $restApi;
        $this->groupManager = $groupManager;
	}

    /**
     * @param Event $event
     */
	public function handle(Event $event): void {
		if (!($event instanceof GroupDeletedEvent)) {
			return;
		}

        $group = $event->getGroup();
        $groupId = $group->getGID();

        try{
            if($this->mapper->findByNcGroups([$groupId])){

                $userIds = [];
                foreach ($group->getUsers() as $user){
                    $userIds[] = $user->getUID();
                }

                $data = json_encode([
                    "group" => $groupId,
                    "users" => $userIds
                ]);

                $result = json_decode($this->restApi->setEndpoint('removeUsersFromGroup')
                    ->setDataBody($data)
                    ->run()
                );

                $groupMap = $this->mapper->find($groupId, $groupId);
                $this->mapper->delete($groupMap);
            }

        } catch (Exception $e){
            $this->logger->error(sprintf('GroupDeletedListener. GroupId: %s. %s',
                $group->getGID(),
                $e->getMessage()),
                ['app' => Application::APP_ID]);
        }
	}
}
