<?php

namespace OCA\VibeEToEe\Listener;

use OCA\VibeEToEe\AppInfo\Application;
use OCA\VibeEToEe\Contracts\IRestApi;
use OCA\VibeEToEe\Events\UnlinkedGroupsEvent;
use OCP\IGroupManager;
use Psr\Log\LoggerInterface;
use OCA\VibeEToEe\Db\GroupMapMapper;
use OCP\EventDispatcher\Event;
use OCP\EventDispatcher\IEventListener;

class UnlinkedGroupsListener implements IEventListener {


    /** @var GroupMapMapper */
    private $mapper;

    /** @var IRestApi */
    private $restApi;

    /** @var LoggerInterface */
    private $logger;

    /** @var IGroupManager */
    private $groupManager;

    /**
     * GroupDeletedListener constructor.
     * @param GroupMapMapper $mapper
     * @param LoggerInterface $logger
     * @param IRestApi $restApi
     * @param IGroupManager $groupManager
     */
    public function __construct(GroupMapMapper $mapper,
                                LoggerInterface $logger,
                                IRestApi $restApi,
                                IGroupManager $groupManager) {
        $this->mapper = $mapper;
        $this->logger = $logger;
        $this->restApi = $restApi;
        $this->groupManager = $groupManager;
    }

    /**
     * @param Event $event
     */
	public function handle(Event $event): void {
		if (!($event instanceof UnlinkedGroupsEvent)) {
			return;
		}

        $groupId = $event->getGroupId();
        $groupVibeId = $event->getVibeGroupId();

		try{
            $group = $this->groupManager->get($groupId);
            $userIds = [];
            foreach ($group->getUsers() as $user){
                $userIds[] = $user->getUID();
            }
            $data = json_encode([
                "group" => $groupId,
                "users" => $userIds
            ]);

            $result = json_decode($this->restApi->setEndpoint('removeUsersFromGroup')
                ->setDataBody($data)
                ->run()
            );


        } catch (\Exception $e){
            $this->logger->error(sprintf('UnlinkedGroupsListener. GroupId: %s',
                $groupId ?? 'undefined'),
                ['app' => Application::APP_ID]);
        }
	}
}
