<?php

namespace OCA\VibeEToEe\Listener;

use Exception;
use OCA\VibeEToEe\AppInfo\Application;
use Psr\Log\LoggerInterface;
use OCA\VibeEToEe\Db\GroupMapMapper;
use OCP\EventDispatcher\Event;
use OCP\EventDispatcher\IEventListener;
use OCP\Group\Events\UserAddedEvent;
use OCA\VibeEToEe\Db\UserAdminMapper as UserMapper;
use OCP\IConfig;
use OCA\VibeEToEe\Contracts\IRestApi;
use OCP\IGroupManager;

class UserAddedToGroupListener implements IEventListener {

    /** @var IGroupManager */
    private $groupManager;

    /** @var GroupMapMapper  */
    private $mapper;

    /** @var LoggerInterface */
    private $logger;

    /** @var UserMapper */
    private $userMapper;

    /** @var IRestApi */
    private $restApi;

    /** @var IConfig */
    private $config;

    private $configVibe;

    /**
     * UserAddedToGroupListener constructor.
     * @param GroupMapMapper $mapper
     * @param LoggerInterface $logger
     * @param UserMapper $userMapper
     * @param IConfig $config
     * @param IRestApi $restApi
     * @param IGroupManager $groupManager
     */
	public function __construct(GroupMapMapper $mapper,
                                LoggerInterface $logger,
                                UserMapper $userMapper,
                                IConfig $config,
                                IRestApi $restApi,
                                IGroupManager $groupManager) {
        $this->mapper = $mapper;
        $this->logger = $logger;
        $this->userMapper = $userMapper;
        $this->configVibe = Application::getConfigVibe();
        $this->config = $config;
        $this->restApi = $restApi;
        $this->groupManager = $groupManager;
	}

    /**
     * @param Event $event
     */
	public function handle(Event $event): void {
		if (!($event instanceof UserAddedEvent)) {
			return;
		}

		$user = $event->getUser();
		$group = $event->getGroup();

		try{
		    // Check, the group in mapper
            if($this->mapper->findByNcGroups([$group->getGID()])){

                // Check, the user with the token exists
                if($this->userMapper->userFind($user->getUID())){

                    $data = json_encode([
                        "group" => $group->getGID(),
                        "users" => [
                            $user->getUID(),
                        ]
                    ]);

                    $result = json_decode($this->restApi->setEndpoint('addUsersToGroup')
                        ->setDataBody($data)
                        ->run()
                    );
                }
                // if user has no token
                else{
                    $this->addUser($user);
                }
            }

        } catch (Exception $e){
            $this->logger->error(sprintf('UserAddedToGroupListener. UserId: %s, GroupId: %s. %s',
                $user->getUID(),
                $group->getGID(),
                $e->getMessage()),
                ['app' => Application::APP_ID]);
        }
	}

	protected function addToGroup(){

    }

    /**
     * @param $user
     * @throws Exception
     */
    protected function addUser($user){
	    try {
            $groups = $this->groupManager->getUserGroups($user);
            $gids = [];
            foreach ($groups as $group) {
                $gids[] = $group->getGID();
            }

            $prepareData = json_encode([
                "encryptionMethod" => "none",
                "encryptedData" => [
                    "users" => [[
                        'identity' => $user->getUID(),
                        'email' => $user->getEMailAddress(),
                        'groups' => (array)$gids
                    ]]
                ]
            ]);

            $result = $this->restApi->setEndpoint('importUserList')
                ->setDataBody($prepareData)
                ->run();

            $data = json_decode($result);

            // TODO: duplicate from /lib/Service/ExportVibe.php parseResultServer
            foreach ($data->result as $u) {
                $this->userMapper->userCreateToken($u->identity, $u->token);
            }


        } catch (Exception $e){
            throw $e;
        }

    }
}
