<?php

namespace OCA\VibeEToEe\Listener;

use OCP\EventDispatcher\Event;
use OCP\EventDispatcher\IEventListener;
use OCP\User\Events\UserChangedEvent;
use OCP\IUserManager;
use OC\Accounts\AccountManager;
use OCP\Accounts\IAccountManager;
use OCA\VibeEToEe\Contracts\IRestApi;

class UserChangedListener implements IEventListener {

    private $userManager;
    /**
     * @var AccountManager
     */
    private $accountManager;

    /** @var IRestApi */
    private $restApi;

    public function __construct(
        IUserManager $userManager,
        IRestApi $restApi,
        AccountManager $accountManager
    ) {

        $this->userManager = $userManager;
        $this->accountManager = $accountManager;
        $this->restApi = $restApi;
    }

    /**
     * @param Event $event
     */
	public function handle(Event $event): void {
		if (!($event instanceof UserChangedEvent)) {
			return;
		}

        $user = $event->getUser();

        // Find the data
        $data = [];
        $userAccount = (array)$this->accountManager->getAccount($user)->getProperties();

        $data['id'] = $user->getUID();
        $data[AccountManager::PROPERTY_EMAIL] = $user->getEMailAddress();
        $data[AccountManager::PROPERTY_DISPLAYNAME] = $user->getDisplayName();
        $data[AccountManager::PROPERTY_PHONE] = $userAccount[IAccountManager::PROPERTY_PHONE]->getValue();
        $data[AccountManager::PROPERTY_ADDRESS] = $userAccount[IAccountManager::PROPERTY_ADDRESS]->getValue();
        $data[AccountManager::PROPERTY_WEBSITE] = $userAccount[IAccountManager::PROPERTY_WEBSITE]->getValue();
        $data[AccountManager::PROPERTY_TWITTER] = $userAccount[IAccountManager::PROPERTY_TWITTER]->getValue();

//        $data['lastLogin'] = $user->getLastLogin() * 1000;
//        $data['backend'] = $user->getBackendClassName();
//        $data['subadmin'] = $this->getUserSubAdminGroupsData($user->getUID());
//        $data['quota'] = $this->fillStorageInfo($user->getUID());
//        $data['groups'] = $gids;
//        $data['language'] = $this->l10nFactory->getUserLanguage($user);
//        $data['locale'] = $this->config->getUserValue($user->getUID(), 'core', 'locale');

        $_data = [];
        foreach ($data as $k => $i){
            array_push($_data, [$k => $i]);
        }

        $sendData = json_encode([
            "encryptionMethod" => "none",
            "encryptedData" => [
                "user" => [
                    "identity" => $data['id'],
                    "fields" => $_data,
                ]
            ]
        ]);

        $this->restApi->setEndpoint('vibeUpdateUserData')
            ->setDataBody($sendData)
            ->run();

	}
}
