<?php

declare(strict_types=1);

namespace OCA\VibeEToEe\Listener;

use OCA\VibeEToEe\Contracts\IKeyStorage;
use OCA\VibeEToEe\Db\UserAdminMapper as UserMapper;
use OCP\EventDispatcher\Event;
use OCP\EventDispatcher\IEventListener;
use OCP\User\Events\UserDeletedEvent;

class UserDeletedListener implements IEventListener {

	private $keyStorage;

    /** @var UserMapper */
    private $userMapper;

	public function __construct(IKeyStorage $keyStorage, UserMapper $userMapper) {
		$this->keyStorage = $keyStorage;
        $this->userMapper = $userMapper;
	}

    /**
     * @param Event $event
     * @throws \Exception
     */
	public function handle(Event $event): void {
		if (!($event instanceof UserDeletedEvent)) {
			return;
		}

        $user = $event->getUser();
		$this->keyStorage->deleteUserKeys($user);
        $this->userMapper->userDelete($user->getUID());
	}
}
