<?php

namespace OCA\VibeEToEe\Listener;

use Exception;
use OCA\VibeEToEe\AppInfo\Application;
use Psr\Log\LoggerInterface;
use OCA\VibeEToEe\Db\GroupMapMapper;
use OCP\EventDispatcher\Event;
use OCP\EventDispatcher\IEventListener;
use OCP\Group\Events\UserRemovedEvent;
use OCA\VibeEToEe\Contracts\IRestApi;

class UserRemovedFromGroupListener implements IEventListener {

    /** @var GroupMapMapper  */
    private $mapper;

    /** @var LoggerInterface */
    private $logger;

    /** @var IRestApi */
    private $restApi;

    /**
     * UserRemovedFromGroupListener constructor.
     * @param GroupMapMapper $mapper
     * @param LoggerInterface $logger
     */
	public function __construct(GroupMapMapper $mapper,
                                LoggerInterface $logger,
                                IRestApi $restApi) {
        $this->mapper = $mapper;
        $this->logger = $logger;
        $this->restApi = $restApi;
	}

    /**
     * @param Event $event
     */
	public function handle(Event $event): void {
		if (!($event instanceof UserRemovedEvent)) {
			return;
		}

		$user = $event->getUser();
		$group = $event->getGroup();

		try{
            if($this->mapper->findByNcGroups([$group->getGID()])){

                $data = json_encode([
                    "group" => $group->getGID(),
                    "users" => [
                        $user->getUID(),
                    ]
                ]);
                $result = json_decode($this->restApi->setEndpoint('removeUsersFromGroup')
                    ->setDataBody($data)
                    ->run()
                );
            }

        } catch (Exception $e){
            $this->logger->error(sprintf('UserRemovedFromGroupListener. UserId: %s, GroupId: %s. %s',
                $user->getUID(),
                $group->getGID(),
                $e->getMessage()),
                ['app' => Application::APP_ID]);
        }
	}
}
