<?php

declare(strict_types=1);

namespace OCA\VibeEToEe\Middleware;

use OCA\VibeEToEe\UserAgentManager;
use OCP\AppFramework\Middleware;
use OCP\AppFramework\OCS\OCSForbiddenException;
use OCP\AppFramework\Utility\IControllerMethodReflector;
use OCP\IRequest;


class UserAgentCheckMiddleware extends Middleware {

	private $reflector;
	private $request;
	private $userAgentManager;

	public function __construct(IControllerMethodReflector $reflector,
								IRequest $request,
								UserAgentManager $userAgentManager) {
		$this->reflector = $reflector;
		$this->request = $request;
		$this->userAgentManager = $userAgentManager;
	}

    /**
     * @param \OCP\AppFramework\Controller $controller
     * @param string $methodName
     * @throws OCSForbiddenException
     */
	public function beforeController($controller, $methodName): void {
		parent::beforeController($controller, $methodName);

		$userAgent = $this->request->getHeader('user-agent');
		if ($this->reflector->hasAnnotation('E2ERestrictUserAgent')
			&& !$this->userAgentManager->supportsVibeEToEe($userAgent)) {
			throw new OCSForbiddenException('Client "' . $userAgent . '" is not allowed to access vibe-e2ee encrypted content.');
		}
	}
}
