<?php

declare(strict_types=1);

namespace OCA\VibeEToEe\Migration;

use OCA\VibeEToEe\Db\UserAdminMapper;
use Psr\Log\LoggerInterface;
use OCP\IUserManager;
use OCA\VibeEToEe\AppInfo\Application;

use OCP\Migration\IRepairStep;
use OCP\Migration\IOutput;


class UsersChk implements IRepairStep {
    private $logger;
    private $userManager;
    private $userAdminMapper;

    const USER_BATCH_SIZE = 200;

    public function __construct(UserAdminMapper $userAdminMapper, LoggerInterface $logger, IUserManager $userManager ){
        $this->logger = $logger;
        $this->userManager = $userManager;
        $this->userAdminMapper = $userAdminMapper;
    }

    /*
     * @inheritdoc
     */
    public function getName()
    {
        return 'Checking and preparation of users';
    }

    /**
     * @inheritdoc
     */
    public function run(IOutput $output): void
    {
        try{
//            $amountUsers = $this->userManager->countUsers(true);
            $amountUsers = array_reduce($this->userManager->countUsers(), function ($v, $w) {
                return $v + (int)$w;
            }, 0);

            $totalSteps = ceil($amountUsers / self::USER_BATCH_SIZE);

            for($i = 0; $i < $totalSteps; $i++){

                foreach ($this->userManager->search('', self::USER_BATCH_SIZE, $i * self::USER_BATCH_SIZE ) as $user) {
                    $this->userAdminMapper->userFindOrCreate($user->getUID());
                }
            }

        } catch (\Exception $e){
            $this->logger->error('Error Checking and preparation of users. ' . $e->getMessage(), ['app' => Application::APP_ID]);
        }
    }
}