<?php

declare(strict_types=1);

namespace OCA\VibeEToEe\Migration;

use Closure;
use Doctrine\DBAL\Types\Type;
use OCP\DB\ISchemaWrapper;
use OCP\Migration\IOutput;
use OCP\Migration\SimpleMigrationStep;


class Version1005Date20200312102456 extends SimpleMigrationStep {

	/**
	 * @param IOutput $output
	 * @param Closure $schemaClosure The `\Closure` returns a `ISchemaWrapper`
	 * @param array $options
	 * @return null|ISchemaWrapper
	 */
	public function changeSchema(IOutput $output, Closure $schemaClosure, array $options) {
		/** @var ISchemaWrapper $schema */
		$schema = $schemaClosure();

		if (!$schema->hasTable('vibee2ee_encryption_lock')) {
			$table = $schema->createTable('vibee2ee_encryption_lock');
			// Id refers to the file id of the locked folder
			// So we don't want autoincrement
			$table->addColumn('id', 'bigint', [
				'notnull' => true,
				'length' => 11,
				'unsigned' => true,
			]);
			$table->addColumn('timestamp', 'bigint', [
				'notnull' => true,
				'length' => 11,
				'unsigned' => true,
			]);
			$table->addColumn('token', 'string', [
				'notnull' => true,
				'length' => 256,
			]);
			$table->addUniqueIndex(['id'], 'vibee2ee_unique_lock');
		}

		return $schema;
	}
}
