<?php

declare(strict_types=1);

namespace OCA\VibeEToEe;

use OCP\Files\Config\IUserMountCache;
use OCP\Files\Folder;
use OCA\VibeEToEe\AppInfo\Application;
use OCA\VibeEToEe\Db\LockMapper;
use OCP\Files\IRootFolder;
use OCP\ILogger;

class RollbackService {

	private $lockMapper;
	private $metaDataStorage;
	private $fileService;
	private $userMountCache;
	private $rootFolder;
	private $logger;

	public function __construct(LockMapper $lockMapper,
								IMetaDataStorage $metaDataStorage,
								FileService $fileService,
								IUserMountCache $userMountCache,
								IRootFolder $rootFolder,
								ILogger $logger) {
		$this->lockMapper = $lockMapper;
		$this->metaDataStorage = $metaDataStorage;
		$this->fileService = $fileService;
		$this->userMountCache = $userMountCache;
		$this->rootFolder = $rootFolder;
		$this->logger = $logger;
	}

    /**
     * @param int $olderThanTimestamp
     * @param int|null $limit
     * @throws \OCP\Files\InvalidPathException
     * @throws \OCP\Files\NotFoundException
     */
	public function rollbackOlderThan(int $olderThanTimestamp, ?int $limit=null): void {
		$locks = $this->lockMapper->findAllLocksOlderThan($olderThanTimestamp, $limit);

		foreach ($locks as $lock) {
			$mountPoints = $this->userMountCache->getMountsForFileId($lock->getId());
			if (empty($mountPoints)) {
				continue;
			}

			$userId = $mountPoints[0]->getUser()->getUID();
			try {
				$userFolder = $this->rootFolder->getUserFolder($userId);
			} catch (\Exception $ex) {
				$this->logger->logException($ex, [
					'app' => Application::APP_ID,
				]);
				continue;
			}

			$nodes = $userFolder->getById($lock->getId());
			if (!isset($nodes[0]) || !$nodes[0] instanceof Folder) {
				continue;
			}

			$node = $nodes[0];
			if ($node->getMTime() > $olderThanTimestamp) {
				continue;
			}

			try {
				$this->fileService->revertChanges($node);
				$this->metaDataStorage->deleteIntermediateFile($userId, $lock->getId());
			} catch (\Exception $ex) {
				$this->logger->logException($ex, [
					'app' => Application::APP_ID,
				]);
				continue;
			}

			$this->lockMapper->delete($lock);
		}
	}
}
