<?php

namespace OCA\VibeEToEe\Service;

use OCP\IUserManager;
use OCP\Migration\IOutput;
use OCP\Migration\IRepairStep;
use Psr\Log\LoggerInterface;
use OCA\VibeEToEe\AppInfo\Application;
use OCA\VibeEToEe\Db\UserAdminMapper;

class ChkUsers  implements IRepairStep
{
    const USER_BATCH_SIZE = 200;

    /** @var LoggerInterface */
    private $logger;

    /** @var IUserManager */
    private $userManager;

    /** @var UserAdminMapper */
    private $userMapper;

    public function __construct(LoggerInterface $logger,
                                IUserManager $userManager,
                                UserAdminMapper $userMapper){
        $this->logger = $logger;
        $this->userManager = $userManager;
        $this->userMapper = $userMapper;
    }

    public function getName()
    {
        return 'Checking and preparation of users';
    }

    public function run(IOutput $output)
    {
        try{
//            $amountUsers = $this->userManager->countUsers(true);
            $amountUsers = array_reduce($this->userManager->countUsers(), function ($v, $w) {
                return $v + (int)$w;
            }, 0);

            $totalSteps = ceil($amountUsers / self::USER_BATCH_SIZE);

            for($i = 0; $i < $totalSteps; $i++){

                foreach ($this->userManager->search('', self::USER_BATCH_SIZE, $i * self::USER_BATCH_SIZE ) as $user) {
                    $this->userMapper->userFindOrCreate($user->getUID());
                }
            }

        } catch (\Exception $e){
            $this->logger->error('Error Checking and preparation of users. ' . $e->getMessage(), ['app' => Application::APP_ID]);
        }
    }
}