<?php

namespace OCA\VibeEToEe\Service;

use Exception;
use GuzzleHttp\Client;
use GuzzleHttp\Exception\ClientException;
use OCA\VibeEToEe\AppInfo\Application;
use OCA\VibeEToEe\Contracts\IRestApi;
use OCP\IConfig;
use Psr\Log\LoggerInterface;

class RestApi implements IRestApi
{
    const TEST_ENDPOINT = '';

    /** @var IConfig */
    protected $config;

    /** @var array */
    public $configVibe;

    /** @var LoggerInterface */
    private $logger;

    protected $endpointURL;
    protected $endpointMETHOD = 'GET';
    protected $endpointHTTPHEADER = null;
    protected $dataQuery = [];
    protected $dataBody = null;

    public function __construct(IConfig $config,
                                LoggerInterface $logger){
        $this->config = $config;
        $this->configVibe = Application::getConfigVibe();
        $this->logger = $logger;
    }

    /**
     * @param string $point
     * @return $this
     * @throws Exception
     */
    public function setEndpoint(string $point){
        $points = $this->configVibe->apiPoints;

        if(!key_exists($point, (array)$points)) {
            throw new Exception('Point undefined');
        }

        $this->endpointURL = $points->base . $points->$point->point;
        $this->endpointMETHOD = $points->$point->method;

        return $this;
    }

    /**
     * @param string $url
     * @param string|null $method
     * @return $this
     */
    public function setEndpointUrl(string $url, string $method = null){
        $this->endpointURL = $url;

        if($method)
            $this->endpointMETHOD = $method;

        return $this;
    }

    /**
     * @param string $method
     * @return $this
     */
    public function setEndpointMETHOD(string $method){
        $this->endpointMETHOD = $method;
        return $this;
    }

    /**
     * @throws Exception
     */
    public function run(){
        try{
            return $this->runCurl();

        } catch (Exception $e){
            throw $e;
        }

    }

    /**
     * @throws Exception
     */
    public function chkAccessApiKey(){
        try{
            $result = $this->setEndpoint('getGroups')
            ->run()
            ;

            return true;

        } catch (Exception $e){
            throw $e;
        }
    }

    /**
     * @param array $data
     * @return $this
     */
    public function setDataQuery(array $data){
        $this->dataQuery = $data;
        return $this;
    }

    /**
     * @param array $data
     * @return $this
     */
    public function setDataBody($data){
        $this->dataBody = $data;
        return $this;
    }

    /**
     * @return false|string
     * @throws Exception
     */
    protected function runCurl(){
        try {
//            $headers = [
//                'cache-control: max-age=0',
//                'upgrade-insecure-requests: 1',
//                'user-agent: Mozilla/5.0 (Windows NT 6.1) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/78.0.3904.97 Safari/537.36',
//                'sec-fetch-user: ?1',
//                'accept: text/html,application/xhtml+xml,application/xml;q=0.9,image/webp,image/apng,*/*;q=0.8,application/signed-exchange;v=b3',
//                'x-compress: null',
//                'sec-fetch-site: none',
//                'sec-fetch-mode: navigate',
//                'accept-encoding: deflate, br',
//                'accept-language: ru-RU,ru;q=0.9,en-US;q=0.8,en;q=0.7',
//            ];
//
//            $ch = curl_init($this->endpointURL);
//            curl_setopt($ch, CURLOPT_COOKIEFILE, __DIR__ . '/cookie.txt');
//            curl_setopt($ch, CURLOPT_COOKIEJAR, __DIR__ . '/cookie.txt');
//            curl_setopt($ch, CURLOPT_HTTPHEADER, $headers);
//            curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
//            curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, false);
//            curl_setopt($ch, CURLOPT_HEADER, true);
//
//
//            $response = curl_exec($ch);
//            $err = curl_error($ch);
//            curl_close($ch);
//
//            if ($err) {
//                throw new Exception("cURL Error #:" . $err);
//            } else {
//                return $response;
//            }
            $client = new Client([
                'cookies' => true,
                'verify' => true,
                'headers' => [
                    'vibe-api-token' => $this->getApiKey(),
                    'Content-Type' => 'application/json',
                    'Accept' => 'application/json'
                ],

            ]);

            if($this->endpointMETHOD == 'GET') {
                $res = $client->request($this->endpointMETHOD, $this->endpointURL, $this->dataQuery);
            }
            elseif($this->endpointMETHOD == 'POST') {
                $res = $client->request($this->endpointMETHOD, $this->endpointURL, [
                    'body' => $this->dataBody
                ]);
            }
            else {
                $res = $client->request($this->endpointMETHOD, $this->endpointURL);
            }

            if($res->getStatusCode() === 200){
                $body = $res->getBody();
                return $body->getContents();
            }

            return false;

        } catch (ClientException $e) {
            $res = $e->getResponse();
            $resRest = json_decode($res->getBody()->getContents());
            if($res->getStatusCode() == 401){
                $this->logger->error(
                    sprintf('From WAMC WEB API. Status code: %s, message: %s', $res->getStatusCode(), $resRest->errorMessage), ['app' => Application::APP_ID]
                );
                throw new Exception($resRest->errorMessage);
            } else {
                $this->logger->error(
                    sprintf('From WAMC WEB API. Status code: %s (%s)', $res->getStatusCode(), $resRest->errorMessage), ['app' => Application::APP_ID]
                );
                throw new Exception($resRest->errorMessage);
            }

        }
        catch (Exception $e){
            $this->logger->error($e->getMessage(), ['app' => Application::APP_ID]);
//            die('<pre>' . print_r([$this->dataBody, $e->getMessage()], true) . '</pre>');
            throw $e;
        }
    }

    /**
     * @return string
     */
    protected function getApiKey(): string{
        return $this->config->getAppValue(Application::APP_ID, 'vibe-apikey');
    }
}
