<?php
namespace OCA\VibeEToEe\Service;

use Exception;

use OC\Accounts\AccountManager;
use OC_Helper;
use OCP\Accounts\IAccountManager;
use OCP\AppFramework\Db\DoesNotExistException;
use OCP\AppFramework\Db\MultipleObjectsReturnedException;

use OCA\VibeEToEe\Db\UserAdmin;
use OCA\VibeEToEe\Db\UserAdminMapper;

use OCP\AppFramework\OCS\OCSException;
use OCP\AppFramework\OCS\OCSNotFoundException;
use OCP\IUserSession;
use OC;
use OCP\Files\NotFoundException;
use OCP\IUserManager;
use OCP\IGroupManager;
use OCP\IConfig;
use OCP\L10N\IFactory;
use OC\User\Backend;

class UserAdminService{

    private $mapper;
    private $userId;
    protected $user;
    private $userManager;
	private $groupManager;
    protected $userVibe = null;
    protected $config;
	protected $accountManager;
	protected $l10nFactory;

    public function __construct(UserAdminMapper $mapper,
								AccountManager $accountManager,
								IConfig $config,
								IFactory $l10nFactory,
                                IUserManager $userManager,
								IGroupManager $groupManager)
    {
        $this->mapper = $mapper;
        $this->userManager = $userManager;
		$this->groupManager = $groupManager;
		$this->config = $config;
		$this->accountManager = $accountManager;
		$this->l10nFactory = $l10nFactory;
        $this->getCurrentUser();
    }

    /**
     * @return bool
     */
    public function isAdmin(){
//        try{
//            $user = $this->mapper->userFindOrCreate($this->user->getUid());
//            $this->userVibe = $user;
//
//            if($user && $user->getPassPhrase() && $user->getVibeKey()) {
//                return true;
//            }
//
//
//        } catch (Exception $e){
//        }

        return true;
    }

    /**
     * @return mixed
     */
    public function getUserVibe(){
        return $this->userVibe;
    }

    /**
     * @param string $url
     * @return string
     * @throws OCSException
     * @throws OCSNotFoundException
     */
    public function urlRegistration(string $url)
    {
        try {
//            $user = $this->userSession->getUser();
            $user = $this->getInfoUser($this->user->getUID());
            $collectInfo = [
                'userId' => $user['id'],
                'email' => $user['email'],
                'displayname' => $user['displayname'],
                'phone' => $user['phone'],
                'address' => $user['address'],
                'website' => $user['website'],
                'twitter' => $user['twitter'],
                'language' => $user['language'],
            ];
            $params = '?';
            foreach ($collectInfo as $k => $v) {
                $params .= sprintf('%s=%s', $k, urlencode($v)) . "&";
            }
//            $params .= 'passPhrase=' . $this->userVibe->getPassPhrase();
            return $url . $params;
        } catch (OC\OCS\Exception $e){
            throw $e;
        }
    }

    /**
     * @param null $userId
     * @param null $passPhrase
     * @param null $vibeKey
     * @return bool
     * @throws Exception
     */
    public function chkResponseVibe($userId = '', $passPhrase = '', $vibeKey = ''){
        $user = $this->mapper->userFind((string)$userId);
        if(!$user)
            throw new Exception('User Not Found');

        if($user && $user->getPassPhrase() !== $passPhrase)
            throw new Exception('passPhrase for this user is not correct');

        if(empty($vibeKey))
            throw new Exception('vibeKey should not be empty');

        $user->setVibeKey($vibeKey);
        $this->mapper->update($user);

        return true;
    }

    public function resetVibeAdmin($userId, $passPhrase){
        try {
            $user = $this->mapper->userFind((string)$userId);
            if($user && $user->getPassPhrase() !== $passPhrase)
                throw new Exception('passPhrase for this user is not correct');

            $user->setVibeKey('');
            $this->mapper->update($user);

            return true;

        } catch (Exception $e){
            throw $e;
        }
    }

    /**
     * @param string $userId
     * @return array
     * @throws OCSException
     * @throws OCSNotFoundException
     */
    public function getInfoUser(string $userId){
		$data = [];

		// Check if the target user exists
		$user = $this->userManager->get($userId);
		if ($user === null) {
			throw new OCSNotFoundException('User does not exist');
		}

		// Should be at least Admin Or SubAdmin!
		if ($this->groupManager->isAdmin($user->getUID())
			|| $this->groupManager->getSubAdmin()->isUserAccessible($user, $user)) {
			$data['enabled'] = $this->config->getUserValue($user->getUID(), 'core', 'enabled', 'true') === 'true';
		} else {
			// Check they are looking up themselves
			if ($user->getUID() !== $user->getUID()) {
				return $data;
			}
		}

		// Get groups data
		$userAccount = (array)$this->accountManager->getAccount($user)->getProperties();
		$groups = $this->groupManager->getUserGroups($user);
		$gids = [];
		foreach ($groups as $group) {
			$gids[] = $group->getGID();
		}

		try {
			$data['storageLocation'] = $user->getHome();
		} catch (NoUserException $e) {
			throw new OCSNotFoundException($e->getMessage(), $e);
		}

		// Find the data
		$data['id'] = $user->getUID();
		$data['lastLogin'] = $user->getLastLogin() * 1000;
		$data['backend'] = $user->getBackendClassName();
		$data['subadmin'] = $this->getUserSubAdminGroupsData($user->getUID());
		$data['quota'] = $this->fillStorageInfo($user->getUID());
		$data[AccountManager::PROPERTY_EMAIL] = $user->getEMailAddress();
		$data[AccountManager::PROPERTY_DISPLAYNAME] = $user->getDisplayName();
		$data[AccountManager::PROPERTY_PHONE] = $userAccount[IAccountManager::PROPERTY_PHONE]->getValue();
		$data[AccountManager::PROPERTY_ADDRESS] = $userAccount[IAccountManager::PROPERTY_ADDRESS]->getValue();
		$data[AccountManager::PROPERTY_WEBSITE] = $userAccount[IAccountManager::PROPERTY_WEBSITE]->getValue();
		$data[AccountManager::PROPERTY_TWITTER] = $userAccount[IAccountManager::PROPERTY_TWITTER]->getValue();
		$data['groups'] = $gids;
		$data['language'] = $this->l10nFactory->getUserLanguage($user);
		$data['locale'] = $this->config->getUserValue($user->getUID(), 'core', 'locale');

		$backend = $user->getBackend();
		$data['backendCapabilities'] = [
			'setDisplayName' => $backend instanceof ISetDisplayNameBackend || $backend->implementsActions(Backend::SET_DISPLAYNAME),
			'setPassword' => $backend instanceof ISetPasswordBackend || $backend->implementsActions(Backend::SET_PASSWORD),
		];


        return $data;
    }

    /**
     * @return $this
     */
    protected function getCurrentUser(){
        $this->user = OC::$server[IUserSession::class]->getUser();
        return $this;
    }


    /**
	 * Get the groups a user is a subadmin of
	 *
	 * @param string $userId
	 * @return array
	 * @throws OCSException
	 */
	protected function getUserSubAdminGroupsData(string $userId): array {
		$user = $this->userManager->get($userId);
		// Check if the user exists
		if ($user === null) {
			throw new OCSNotFoundException('User does not exist');
		}

		// Get the subadmin groups
		$subAdminGroups = $this->groupManager->getSubAdmin()->getSubAdminsGroups($user);
		$groups = [];
		foreach ($subAdminGroups as $key => $group) {
			$groups[] = $group->getGID();
		}

		return $groups;
	}

    /**
     * @param string $userId
     * @return array
     * @throws OCSException
     */
	protected function fillStorageInfo(string $userId): array {
		try {
			\OC_Util::tearDownFS();
			\OC_Util::setupFS($userId);
			$storage = OC_Helper::getStorageInfo('/');
			$data = [
				'free' => $storage['free'],
				'used' => $storage['used'],
				'total' => $storage['total'],
				'relative' => $storage['relative'],
				'quota' => $storage['quota'],
			];
		} catch (NotFoundException $ex) {
			// User fs is not setup yet
			$user = $this->userManager->get($userId);
			if ($user === null) {
				throw new OCSException('User does not exist', 101);
			}
			$quota = $user->getQuota();
			if ($quota !== 'none') {
				$quota = OC_Helper::computerFileSize($quota);
			}
			$data = [
				'quota' => $quota !== false ? $quota : 'none',
				'used' => 0
			];
		}
		return $data;
	}
}
