<?php

namespace OCA\VibeEToEe\Settings;

use OCP\IL10N;
use OCP\IURLGenerator;
use OCP\Settings\IIconSection;
use OCA\VibeEToEe\AppInfo\Application;

class AdminSection implements IIconSection
{
    const APP_ID = 'vibeetoee';
    const APP_NAME = 'VIBE E2E Encryption';

    /** @var IL10N */
    private $l;

    /** @var IURLGenerator */
    private $urlGenerator;

    public function __construct(IL10N $l, IURLGenerator $urlGenerator)
    {
        $this->l = $l;
        $this->urlGenerator = $urlGenerator;
    }

    /**
     * returns the ID of the section. It is supposed to be a lower case string,
     * e.g. 'ldap'
     *
     * @returns string
     */
    public function getID()
    {
        return Application::APP_ID;
    }

    /**
     * returns the translated name as it should be displayed, e.g. 'LDAP / AD
     * integration'. Use the L10N service to translate it.
     *
     * @return string
     */
    public function getName()
    {
//        return $this->l->t(self::APP_NAME);
        return Application::APP_NAME;
    }

    /**
     * @return int whether the form should be rather on the top or bottom of
     * the settings navigation. The sections are arranged in ascending order of
     * the priority values. It is required to return a value between 0 and 99.
     *
     * E.g.: 70
     */
    public function getPriority()
    {
        return 80;
    }

    /**
     * @return The relative path to a an icon describing the section
     */
    public function getIcon()
    {
        return $this->urlGenerator->imagePath(Application::APP_ID, 'icon.svg');
    }
}
