<?php

namespace OCA\VibeEToEe\Settings;

use OCP\AppFramework\Http\TemplateResponse;
use OCP\BackgroundJob\IJobList;
use OCP\IConfig;
use OCP\IDateTimeFormatter;
use OCP\IL10N;
use OCP\Settings\ISettings;
use OCA\VibeEToEe\Service\UserAdminService as UAservice;
use OCA\VibeEToEe\AppInfo\Application;

class AdminSettings implements ISettings
{
    /** @var IConfig */
    private $config;

    /** @var IL10N */
    private $l;

    /** @var IDateTimeFormatter */
    private $dateTimeFormatter;

    /** @var IJobList */
    private $jobList;

    private $uaservice;

    private $appInfo;


    public function __construct(IConfig $config,
                                IL10N $l,
                                IDateTimeFormatter $dateTimeFormatter,
                                IJobList $jobList,
                                UAservice $uaservice,
                                Application $appInfo
    )
    {
        $this->config = $config;
        $this->l = $l;
        $this->dateTimeFormatter = $dateTimeFormatter;
        $this->jobList = $jobList;
        $this->uaservice = $uaservice;
        $this->appInfo = $appInfo;
    }

    /**
     * @return TemplateResponse
     */
    public function getForm()
    {
        $vibeConfig = $this->appInfo->configVibe;

        $parameters = [
            'isAdmin' => true,
            'regUrl' => $this->uaservice->urlRegistration($vibeConfig->urlVibeRegister),
        ];

        return new TemplateResponse(Application::APP_ID, 'admin', $parameters);
    }

    /**
     * @return string the section ID, e.g. 'sharing'
     */
    public function getSection()
    {
        return Application::APP_ID;
    }

    /**
     * @return int whether the form should be rather on the top or bottom of
     * the admin section. The forms are arranged in ascending order of the
     * priority values. It is required to return a value between 0 and 100.
     */
    public function getPriority()
    {
        return 50;
    }

}
