<?php

namespace OCA\VibeEToEe\Settings;

use OCA\VibeEToEe\AppInfo\Application;
use OCP\AppFramework\Http\TemplateResponse;
use OCP\IL10N;
use Psr\Log\LoggerInterface;
use OCP\IUserSession;
use OCP\IGroupManager;
use OCP\Settings\ISettings;
use OCA\VibeEToEe\Db\UserAdminMapper;

class PersonaSettings implements ISettings
{
    /** @var IL10N */
    private $l;

    /** @var LoggerInterface */
    private $logger;

    /** @var IUserSession */
    private $userSession;

    /** @var IGroupManager */
    protected $groupManager;

    /** @var UserAdminMapper */
    private $userMapper;

    public function __construct(IL10N $l,
                                LoggerInterface $logger,
                                IUserSession $userSession,
                                UserAdminMapper $userMapper,
                                IGroupManager $groupManager)
    {
        $this->l = $l;
        $this->logger = $logger;
        $this->userSession = $userSession;
        $this->groupManager = $groupManager;
        $this->userMapper = $userMapper;
    }

    /**
     * @return TemplateResponse
     */
    public function getForm()
    {
        try {
            $user = $this->userSession->getUser();
            $config = Application::getConfigVibe();

            $tokenIssue = false;
            if($tokenUser = $this->userMapper->userFind($user->getUID())){
                $tokenIssue = $tokenUser->getVibeKey();
            }

            $parameters = [
                'userName' => $user->getUID(),
                'isAdmin' => $this->groupManager->isAdmin($user->getUID()),
                'actionURL' => $config->linkVSAR,
                'tokenIssue' => $tokenIssue,
            ];

            return new TemplateResponse(Application::APP_ID, 'personal', $parameters, '');

        } catch (\Exception $e) {
            throw $e;
        }
    }

    /**
     * @return string the section ID, e.g. 'sharing'
     */
    public function getSection()
    {
//        return Application::APP_ID;
		return 'security';
    }

    /**
     * @return int whether the form should be rather on the top or bottom of
     *             the admin section. The forms are arranged in ascending order of the
     *             priority values. It is required to return a value between 0 and 100.
     *
     * E.g.: 70
     */
    public function getPriority()
    {
        return 70;
    }
}
