<?php

declare(strict_types=1);

namespace OCA\VibeEToEe;

use BadMethodCallException;
use OC\Security\IdentityProof\Manager;
use RuntimeException;

class SignatureHandler {

	private $identityProofManager;
	private $validity = 7300;

	public function __construct(Manager $identityProofManager) {
		$this->identityProofManager = $identityProofManager;
	}

    /**
     * @param string $csr
     * @return string
     */
	public function sign(string $csr): string {
		$systemKeys = $this->identityProofManager->getSystemKey();
		$signedCertificate = openssl_csr_sign($csr, null, $systemKeys->getPrivate(), $this->validity);
		if ($signedCertificate === false) {
			throw new BadMethodCallException('could not sign the CSR, please make sure to submit a valid CSR');
		}
		openssl_x509_export($signedCertificate, $result);
		return $result;
	}

    /**
     * @return string
     */
	public function getPublicServerKey(): string {
		$publicKey = $this->identityProofManager->getSystemKey()->getPublic();
		return $publicKey;
	}
}
