<?php

namespace OCA\VibeEToEe;

use OCA\VibeEToEe\Contracts\IStorageIdentify;
use OCP\Files\IAppData;

class StorageIdentify implements IStorageIdentify
{
    protected $user;
    private $identifyFolder = '/identifies';
    private $fileName = '';
    private $filePath = '';
    private $appData;

    /**
     * StorageIdentify constructor.
     * @param IAppData $appData
     */
    public function __construct(IAppData $appData)
    {
        $this->appData = $appData;
    }

    public function getIdentify($userId)
    {
        $this->user = $userId;
        $this->setFileName($this->user);
        if ($this->chkFile()) {
            $fileRoot = $this->appData->getFolder($this->filePath);
            $content = $fileRoot->getFile($this->fileName)->getContent();
            return $content;
        } else {
            throw new \Exception('Identify for this user not found');
        }
    }

    /**
     * @param string $userId
     * @param string $identify
     * @return bool
     * @throws \Exception
     */
    public function setIdentify(string $userId, string $identify)
    {
        try {
            $this->user = $userId;

            $this->setFileName($this->user);

            if ($this->chkFile()) {
                $this->updateIdentify($identify);

            } else {
                $fileRoot = $this->appData->getFolder($this->filePath);
                $fileRoot
                    ->newFile($this->fileName)
                    ->putContent((string)$identify);
            }
            return true;

        } catch (\Exception $e) {
            throw $e;
        }

    }

    /**
     * @param string $strData
     * @return bool
     * @throws \OCP\Files\NotFoundException
     * @throws \OCP\Files\NotPermittedException
     */
    protected function updateIdentify(string $strData)
    {
        $fileRoot = $this->appData->getFolder($this->filePath);
        $fileRoot->getFile($this->fileName)
            ->putContent((string)$strData);

        return true;
    }


    /**
     * @return bool
     * @throws \OCP\Files\NotFoundException
     * @throws \OCP\Files\NotPermittedException
     */
    protected function chkFile(): bool
    {
        $this->verifyFolderStructure($this->filePath);
        $fileRoot = $this->appData->getFolder($this->filePath);
        return $fileRoot->fileExists($this->fileName);
    }

    /**
     * @param string $val
     * @return $this
     */
    protected function setFileName(string $val)
    {
        $this->fileName = md5($val);
        $this->setFilePath($this->fileName);
        return $this;
    }

    /**
     * @param string $val
     * @return $this
     */
    protected function setFilePath(string $val)
    {
        $hash = str_split($val, 3);
        $this->filePath = sprintf('%s%s%s%s%s%s%s%s',
            $this->identifyFolder,
            DIRECTORY_SEPARATOR,
            $hash[0],
            DIRECTORY_SEPARATOR,
            $hash[1],
            DIRECTORY_SEPARATOR,
            $hash[2],
            DIRECTORY_SEPARATOR
        );
        return $this;
    }

    /**
     * @param string $path
     * @throws \OCP\Files\NotFoundException
     * @throws \OCP\Files\NotPermittedException
     */
    protected function verifyFolderStructure(string $path): void
    {
        $keyStorageRoot = $this->appData->getFolder('/');
        if (!$keyStorageRoot->fileExists($path)) {
            $this->appData->newFolder($path);
        }
    }
}
