<?php

declare(strict_types=1);

namespace OCA\VibeEToEe;

use OC\AppFramework\Http\Request;

class UserAgentManager {

	private $supportedUserAgents;

	public function __construct() {
		$this->supportedUserAgents = [
			'/^Mozilla\/5\.0 \(Android\) Nextcloud\-android\/(?<version>(0|[1-9]\d*)\.(0|[1-9]\d*)\.(0|[1-9]\d*)).*$/' => '3.13.0',
			'/^Mozilla\/5\.0 \([A-Za-z ]+\) (mirall|csyncoC)\/(?<version>(0|[1-9]\d*)\.(0|[1-9]\d*)\.(0|[1-9]\d*)).*$/' => '3.0.0',
			'/^Mozilla\/5\.0 \(iOS\) Nextcloud\-iOS\/(?<version>(0|[1-9]\d*)\.(0|[1-9]\d*)\.(0|[1-9]\d*)).*$/' => '3.0.5',
		];
	}

    /**
     * @param string $client
     * @return bool
     */
	public function supportsVibeEToEe(string $client): bool {
		$supportedUAs = $this->getSupportedUserAgents();

		foreach ($supportedUAs as $regex => $minVersion) {
			$doesMatch = preg_match($regex, $client, $matches);
			if ($doesMatch === 0) {
				continue;
			}

			if (empty($minVersion)) {
				return true;
			}
			if (!isset($matches['version'])) {
				return false;
			}

			return (version_compare($matches['version'], $minVersion) > -1);
		}

		return false;
	}

    /**
     * @return array
     */
	protected function getSupportedUserAgents(): array {
		return $this->supportedUserAgents;
	}
}
