import * as types from '../mutation-types'
import HttpClient from '@nextcloud/axios'
import { linkTo } from '@nextcloud/router'

// state
export const state = {
	links: [],
}

// getters
export const getters = {
	links: state => state.links,
}

// mutations
export const mutations = {
	[types.SET_LINKS](state, data) {
		state.links = data
	},
	[types.ADD_LINKS](state, { ids }) {
		state.links.push(ids)
	},
	[types.REMOVE_LINKS](state, { idx }) {
		state.links.splice(idx, 1)
	},
}

// actions
export const actions = {

	async addLinks({ commit, dispatch }, ids) {
		try {
			await HttpClient.post(linkTo('vibeetoee', 'index.php') + '/api/v1/links/create', ids)

		} catch (error) {
			console.debug(error)
			return false
		}

		await dispatch('getLinks')
		return true
	},

	 async removeLinks({ commit, dispatch }, ids) {
		try {
			await HttpClient.post(linkTo('vibeetoee', 'index.php') + '/api/v1/links/remove', ids)

		} catch (error) {
			console.debug(error)
			return false

		} finally {
			await dispatch('getLinks')
		}
		 return true
	},

	async getLinks({ commit }) {
		try {
			const response = await HttpClient.get(linkTo('vibeetoee', 'index.php') + '/api/v1/links/index', {
				headers: { Accept: 'application/json' },
			})

			const res = response.data.ocs

			if (res.meta.status === 'ok') {
				commit(types.SET_LINKS, res.data)
			}
			return true

		} catch (error) {
			console.debug(error)
			return false
		}

	},
}
