import * as types from '../mutation-types'
import HttpClient from '@nextcloud/axios'
import { linkTo } from '@nextcloud/router'

// state
export const state = {
	usersData: [],
	users: [],
	currentPage: 1,
	totalPages: 1,
	totalRows: 1,
}

// getters
export const getters = {
	usersData: state => state.usersData,
	currentPage: state => state.currentPage,
	totalPages: state => state.totalPages,
	totalRows: state => state.totalRows,
	users: state => state.users,
}

// mutations
export const mutations = {
	[types.SET_USERS](state, data) {
		state.usersData = data
		state.users = data.data
		state.currentPage = data.page
		state.totalPages = data.totalPages
		state.totalRows = data.totalRows
	},
	[types.ADD_USERS](state, { ids }) {
		state.users.push(ids)
	},
	[types.REMOVE_USERS](state, { idx }) {
		state.users.splice(idx, 1)
	},
}

// actions
export const actions = {
	async getUsers({ commit }, page) {
		try {
			const response = await HttpClient.get(linkTo('vibeetoee', 'index.php') + '/api/v1/neusers/index',
				{
					params: { page },
					headers: { Accept: 'application/json' },
				},
			)

			const res = response.data.ocs

			if (res.meta.status === 'ok') {
				commit(types.SET_USERS, res.data)
			}
			return true

		} catch (error) {
			console.debug(error)
			return false
		}

	},
}
