<?php

declare(strict_types=1);
/**
 * @copyright Copyright (c) 2020 Georg Ehrke <georg-nextcloud@ehrke.email>
 *
 * @license GNU AGPL version 3 or any later version
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

namespace OCA\VibeEToEe\Tests\Unit\BackgroundJob;

use OCA\VibeEToEe\BackgroundJob\RollbackBckJob;
use OCA\VibeEToEe\RollbackService;
use OCP\AppFramework\Utility\ITimeFactory;
use OCP\IConfig;
use Test\TestCase;

class RollbackBckJobTest extends TestCase {

	/** @var IConfig|\PHPUnit\Framework\MockObject\MockObject */
	private $config;

	/** @var ITimeFactory|\PHPUnit\Framework\MockObject\MockObject */
	private $timeFactory;

	/** @var RollbackService|\PHPUnit\Framework\MockObject\MockObject */
	private $rollbackService;

	/** @var RollbackBckJob */
	private $rollbackBackgroundJob;

	protected function setUp(): void {
		parent::setUp();

		$this->config = $this->createMock(IConfig::class);
		$this->timeFactory = $this->createMock(ITimeFactory::class);
		$this->rollbackService = $this->createMock(RollbackService::class);

		$this->rollbackBackgroundJob = new RollbackBckJob($this->config, $this->timeFactory, $this->rollbackService);
	}

	/**
	 * @dataProvider runDataProvider
	 *
	 * @param string $automaticRollback
	 * @param bool $expectsServiceCall
	 * @param int $automaticRollbackTTL
	 * @param int $expectedTimestamp
	 */
	public function testRun(string $automaticRollback, bool $expectsServiceCall, int $automaticRollbackTTL, int $expectedTimestamp):void {
		$this->config->expects($this->at(0))
			->method('getAppValue')
			->with('end_to_end_encryption', 'automatic_rollback', 'yes')
			->willReturn($automaticRollback);

		if ($expectsServiceCall) {
			$this->config->expects($this->at(1))
				->method('getAppValue')
				->with('end_to_end_encryption', 'automatic_rollback_ttl', 60 * 60 * 24)
				->willReturn($automaticRollbackTTL);

			$this->timeFactory->expects($this->once())
				->method('getTime')
				->willReturn(500);

			$this->rollbackService->expects($this->once())
				->method('rollbackOlderThan')
				->with($expectedTimestamp);
		}

		self::invokePrivate($this->rollbackBackgroundJob, 'run', [[]]);
	}

	public function runDataProvider(): array {
		return [
			['no', false, -1,  -1],
			['yes', true, 60, 440],
			['yes', true, 80, 420],
		];
	}
}
